function v = gramian_comred(n,m)
% gramian_comred ( gramian-based combined reduction )
% by Christian Himpe, 2013 ( http://gramian.de )
% released under BSD 2-Clause License ( http://gramian.de/#license )
%*

warning off all
optflags = optimset('Display','off','MaxIter',10);

%%%%%%%% Setup %%%%%%%%

%% CONSTANTS
 J = n;
 O = J;
 N = n*n;
 R = O;
 t = [0 0.01 1];
 T = (t(3)-t(1))/t(2);
 u = [N*ones(J,1) zeros(J,T-1)];
 X =    ones(N,1);
 v = zeros(5,1);
%%

%% PARAMETER
 A = rand(N,N);
 A(1:N+1:end) = -0.55*N;
 A = 0.5*(A+A');
 B = rand(N,J);
 C = B';
 P = A(:);
 p = -eye(N);
 p = p(:);
%%

%% FUNCTIONS
 F = @(x,u,p) reshape(p,[N N])*tanh(x) + B*u;
 G = @(x,u,p) C*x;
 Y = rk2(F,G,[J N O],t,X,u,P);
%%

%%%%%%%% Nonlinear Combined Reduction %%%%%%%%

%% FULL
 %% ONLINE
  tic;
  o = @(z,w) norm2(Y-rk2(F,G,[J N O],t,X,u,z));
  q = fminunc(o,p,optflags);
  y = rk2(F,G,[J N O],t,X,u,q);
  v(1) = toc;
  v(2) = norm(norm(Y - y)./norm(Y));
 %%
%%

%% WJ
 %% OFFLINE
  tic;
  WJ = emgr(F,G,[J N O],p,t,'j',[1 0 0 0 0 0 0 0 0 0]);
  [UU D VV] = svd(WJ{1}); UU = UU(:,1:R);   VV = VV(:,1:R)';
  [PP D QQ] = svd(WJ{2}); PP = PP(1:R*R,:); QQ = QQ(1:R*R,:)';
  b = VV*B;
  c = C*UU;
  x = VV*X;
  f = @(x,u,p) reshape(p,[R R])*tanh(x) + b*u;
  g = @(x,u,p) c*x;
  v(3) = toc;
 %%

 %% ONLINE
  tic;
  o = @(z,w) norm2(Y-rk2(f,g,[J N O],t,x,u,z));
  q = fminunc(o,PP*p,optflags);
  y = rk2(f,g,[J N O],t,x,u,q);
  v(4) = toc;
  v(5) = norm(norm(Y - y)./norm(Y));
  RELER = abs(Y - y)./abs(Y);
 %%
%%

%% PLOT
 if(nargin<2 || m==0 ) return; end

 cmap = [(1:-0.01:0)',(1:-0.0065:0.35)',(1:-0.0046:0.54)'];
 figure('PaperSize',[2.5,10],'PaperPosition',[0,0,10,2.5]);
 imagesc(RELER); caxis([0 max(max(RELER))]); h=colorbar; colormap(cmap);
 set(gca,'YTick',1:N);
 set(gca,'ycolor',[0.68 0.93 0.93]);
 set(gca,'xcolor',[0.68 0.93 0.93]);
 set(h,'xcolor',[0.68 0.93 0.93]);
 set(h,'ycolor',[0.68 0.93 0.93]);

 xt = get(gca,'XTick');
 xl = cellstr(get(gca,'XTickLabel'));
 set(gca,'XTickLabel',[]);
 xn = numel(xl);
 yl = ylim;
 ix = 1:1:xn;
 tx = text(xt(ix),repmat(yl(2)+0.1,numel(ix),1),xl(ix),'HorizontalAlignment','center','VerticalAlignment','top','color',[0 0.35 0.54],'fontsize',5);

 yt = get(gca,'YTick');
 yl = cellstr(get(gca,'YTickLabel'));
 set(gca,'YTickLabel',[]);
 yn = numel(yl);
 xl = xlim;
 iy = 1:1:yn;
 ty = text(repmat(xl(1)-0.2,numel(iy),1),yt(iy),yl(iy),'HorizontalAlignment','right','VerticalAlignment','middle','color',[0 0.35 0.54],'fontsize',5);

 ct = get(h,'YTick');
 cl = cellstr(get(h,'YTickLabel'));
 set(h,'YTickLabel',[]);
 cn = numel(cl);
 xl = get(h,'xlim');
 ic = 1:1:cn;
 tc = text(repmat(xl(2)+0.2,numel(ic),1),ct(ic),cl(ic),'HorizontalAlignment','left','VerticalAlignment','middle','color',[0 0.35 0.54],'fontsize',5,'parent',h);

 if(m==2)
	print -depsc gramian_comred.eps;
 end
%%

%%%%%%%% RK2 Integration %%%%%%%%

function y = rk2(f,g,q,t,x,u,p)
T = (t(3)-t(1))/t(2);
y = zeros(q(3),T);
h = t(2);

for A=1:T
	x = x + h*f(x + 0.5*h*f(x,u(:,A),p),u(:,A),p); %Improved Eulers Method
	y(:,A) = g(x,u(:,A),p);
end

%%%%%%%% Timeseries 2-Norm %%%%%%%%

function x = norm2(X)
	x = sqrt(sum(sum(X.*X)));
