function v = greedy_comred(n,m)
% greedy_comred (optimization-based combined reduction)
% by Christian Himpe, 2013 ( http://wwwmath.uni-muenster.de/u/himpe/ )
% released under BSD 2-Clause License
%*

warning off all;
optflags = optimset('Display','off','MaxIter',10);

%%%%%%%% Setup %%%%%%%%

%% CONSTANTS
 J = n;
 N = n*n;
 O = J;
 R = O;
 t = [0 0.01 1];
 T = (t(3)-t(1))/t(2);
 u = [N*ones(J,1) zeros(J,T-1)];
 X =    ones(N,1);
 v = zeros(5,1);
%%

%% PARAMETER
 A = rand(N,N);
 A(1:N+1:end) = -0.55*N;

 B = rand(N,J);
 C = rand(O,N);
 P = A(:);
 p = -eye(N);
 p = p(:); 
%%

%% FUNCTIONS
 F = @(x,u,p) reshape(p,[N N])*x+B*u;
 G = @(x,u,p) C*x;
 Y = rk2(F,G,[J N O],t,X,u,P);
%%

%%%%%%%% Greedy Estimation %%%%%%%%

%% FULL
 %%ONLINE
  tic;
  o = @(z,w) norm2(Y-rk2(F,G,[J N O],t,X,u,z));
  q = fminunc(o,p,optflags);
  y = rk2(F,G,[J N O],t,X,u,q);
  v(1) = toc;
  v(2) = norm(norm(Y-y)./norm(Y));
 %%
%%

%% Trust
 %% OFFLINE
  tic;
  [P V] = optr(N,R,B,C,0,X,t,u,[1 1],0,1,Y);
  b = V'*B;
  c = C*V;
  x = V'*X;
  v(3) = toc;
 %%

 %% ONLINE 
  tic;
  f = @(x,u,p) V'*reshape(P*p,[N N])*V*x+b*u;
  g = @(x,u,p) c*x;
  o = @(z,w) norm2(Y-rk2(f,g,[J N O],t,V'*X,u,z));
  q = fminunc(o,P'*p,optflags);
  y = rk2(f,g,[J N O],t,V'*X,u,q);
  v(4) = toc;
  v(5) = norm(norm(Y-y)./norm(Y));
  RELER = abs(Y-y)./abs(Y);
 %%
%%

%% PLOT 
 if(nargin<2 || m==0 ) return; end

 cmap = [(1:-0.01:0)',(1:-0.0065:0.35)',(1:-0.0046:0.54)'];
 figure('PaperSize',[2.5,10],'PaperPosition',[0,0,10,2.5]);
 imagesc(RELER); caxis([0 max(max(RELER))]); h=colorbar; colormap(cmap);
 set(gca,'YTick',1:N);
 set(gca,'ycolor',[0.68 0.93 0.93]);
 set(gca,'xcolor',[0.68 0.93 0.93]);
 set(h,'xcolor',[0.68 0.93 0.93]);
 set(h,'ycolor',[0.68 0.93 0.93]);

 xt = get(gca,'XTick');
 xl = cellstr(get(gca,'XTickLabel'));
 set(gca,'XTickLabel',[]);
 xn = numel(xl);
 yl = ylim;
 ix = 1:1:xn;
 tx = text(xt(ix),repmat(yl(2),numel(ix),1),xl(ix),'HorizontalAlignment','center','VerticalAlignment','top','color',[0 0.35 0.54],'fontsize',5);

 yt = get(gca,'YTick');
 yl = cellstr(get(gca,'YTickLabel'));
 set(gca,'YTickLabel',[]);
 yn = numel(yl);
 xl = xlim;
 iy = 1:1:yn;
 ty = text(repmat(xl(1),numel(iy),1),yt(iy),yl(iy),'HorizontalAlignment','right','VerticalAlignment','middle','color',[0 0.35 0.54],'fontsize',5);

 ct = get(h,'YTick');
 cl = cellstr(get(h,'YTickLabel'));
 set(h,'YTickLabel',[]);
 cn = numel(cl);
 xl = get(h,'xlim');
 ic = 1:1:cn;
 tc = text(repmat(xl(2),numel(ic),1),ct(ic),cl(ic),'HorizontalAlignment','left','VerticalAlignment','middle','color',[0 0.35 0.54],'fontsize',5,'parent',h);

 if(s==2)
	print -depsc greedy_comred.eps;
 end

%%%%%%%% RK2 Integration %%%%%%%%

function y = rk2(f,g,q,t,x,u,p)
T = (t(3)-t(1))/t(2);
y = zeros(q(3),T);
h = t(2);

for A=1:T
	x = x + h*f(x + 0.5*h*f(x,u(:,A),p),u(:,A),p); %Improved Eulers Method
	y(:,A) = g(x,u(:,A),p);
end

%%%%%%%% Timeseries 2-Norm %%%%%%%%

function x = norm2(X)
	x = sqrt(sum(sum(X.*X)));
