function [P V] = optr(N,R,B,C,F,X,t,u,cf,P,S,y)
% optr (Version 0.9)
% by Christian Himpe, 2013 ( http://wwwmath.uni-muenster.de/u/himpe/ )
% released under 2-clause BSD license
%
% About:
%  optr (Optimization-Based Reduction)
%  reduces simultaneously parameters and states of
%  of a dynamic system of the following type:
%
%  x' = Ax + Bu + F
%  y  = Cx
%
%  A in R^NxN
%  B in R^NxM
%  C in R^OxN
%  F in R^N
%
% Parameters:
%  (scalar) N : original state dimension
%  (scalar) R : target reduced state dimension
%  (matrix) B : input  (states x inputs)
%  (matrix) C : output (outputs x states)
%  (vector) F : source (states x 1)
%  (vector) X : initial values (states x 1)
%  (vector) t : timing [start,step,stop]
%  (matrix) u : input data (inputs x timesteps)
%  (vector) cf: configuration [trust,data]
%  (vector) P : prior mean (parameters x 1)
%  (matrix) S : prior covariance matrix (parameters x parameters)
%  (matrix) y : experimental data (outputs x timesteps)
%
% Output:
%  (matrix) P : Parameter Projection (reduced parameters x parameters)
%  (matrix) V : State Projection (reduced states x states)
%*

%Extract Dimensions
T = (t(3)-t(1))/t(2);		%Number of Timesteps
J = size(B,2);			%Number of Inputs
O = size(C,1);			%Number of Outputs
%*

%Fix lazy arguments
if(C==1)        C = speye(N);     end
if(B==0)        B = sparse(N,J);  end
if(numel(F)==1) F = F*ones(N,1);  end
if(numel(X)==1) X = X*ones(N,1);  end
if(nargin<10)   P = 0;            end
if(P==0)        P = -eye(N); P = P(:); end
if(nargin<11)   S = 1;            end
if(S==1)        S = speye(N*N);   end
%*

%Parameter Checks
if(R>=N || R<0)        error('ERROR: Be serious!');                                             end
if(N~=size(B,1))       error('ERROR: Dimension of state and input do not agree in matrix B!');  end
if(N~=size(C,2))       error('ERROR: Dimension of state and output do not agree in matrix C!'); end
if(N~=size(F,1))       error('ERROR: Dimension of state and source do not agree in vector F!'); end
if(N~=size(X,1))       error('ERROR: Dimension of state and initial state do not agree in vector X!'); end
if(J~=size(u,1))       error('ERROR: Inputs do not match input series!');                       end
if(T~=size(u,2))       error('ERROR: Timeseries length does not match input length!');          end
if(size(P)~=[N*N 1])   error('ERROR: Parameter dimensions do not match parameter count!');      end
if(size(S)~=[N*N N*N]) error('ERROR: Covariance dimensions do not match parameter count!');     end
%*

%Parameter Transformation
A = @(p) reshape(p,[sqrt(numel(p)) sqrt(numel(p))]);
%*

%Init Parameter Projection Vector
V = mean(rk1(A(P),B,speye(N),F,t,X,u),2);

switch(cf(1))
	case 0
		p = P;
	case 1
		p = 1;
end
%*

warning off all 
optflag = optimset('Display','off');	%because fminunc talks alot

%Main Loop
for I=1:R
%I
	%apply optimization to current problem
	if(cf(1))
		Q = P;
	else
		Q = 1;
	end

	if(cf(2)~=0&& ~isscalar(y))
		J = @(p) J1(p,A,B,C,F,t,X,u,S,Q,V,y);
	else
		J = @(p) J0(p,A,B,C,F,t,X,u,S,Q,V);
	end

	p = fminunc(J,p,optflag);
	v = rk1(A(Q*p),B,speye(N),F,t,X,u);

	%append projection matrices
	a = p;
	if(cf(1)) a(N*N,1) = 0; p(I+1,1) = 0; end
	P = orth([P a]);
	V = orth([V mean(v,2)]);
%p'
end
%*

%%%%%%%% OBJECTIVE %%%%%%%%
function j = J0(p,A,B,C,F,t,X,u,S,P,V)

	b = V'*B;
	f = V'*F;
	x = V'*X;
	c = C*V;

	q = P*p;

	Y = rk1(A(q),B,C,F,t,X,u);
	y = rk1(V'*A(q)*V,b,c,f,t,x,u);

	j = 0.9*max(sqrt(sum(abs(Y-y).^2,1)))+0.1*(q'*S*q);

function j = J1(p,A,B,C,F,t,X,u,S,P,V,D)

	b = V'*B;
	f = V'*F;
	x = V'*X;
	c = C*V;
%size(P)
%size(p)
	q = P*p;

	y = rk1(V'*A(q)*V,b,c,f,t,x,u);

	j = 0.9*max(sqrt(sum(abs(D-y).^2,1)))+0.1*(q'*S*q);

%%%%%%%% RK1 %%%%%%%%
function y = rk1(A,B,C,F,t,x,u)

T = (t(3)-t(1))/t(2);
h = t(2);
y = zeros(size(C,1),T);

for I=1:T
	x = x + h*(A*x + B*u(:,I) + F);
	y(:,I) = C*x;
end
